@@ -4,7 +4,7 @@ package JSON::Schema::Modern::Vocabulary::FormatAssertion;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Implementation of the JSON Schema Format-Assertion vocabulary
 
-our $VERSION = '0.568';
+our $VERSION = '0.570';
 
 use 5.020;
 use Moo;
@@ -34,9 +34,6 @@ sub keywords {
 {
   # for now, all built-in formats are constrained to the 'string' type
 
-  my $is_datetime = sub {
-    eval { Time::Moment->from_string(uc($_[0])) } ? 1 : 0,
-  };
   my $is_email = sub {
     Email::Address::XS->parse($_[0])->is_valid;
   };
@@ -50,7 +47,7 @@ sub keywords {
     my @o = split(/\./, $_[0], 5);
     @o == 4 && (grep /^(?:0|[1-9][0-9]{0,2})$/, @o) == 4 && (grep $_ < 256, @o) == 4;
   };
-  # https://tools.ietf.org/html/rfc3339#appendix-A with some additions for the 2000 version
+  # https://datatracker.ietf.org/doc/html/rfc3339#appendix-A with some additions for the 2000 version
   # as defined in https://en.wikipedia.org/wiki/ISO_8601#Durations
   my $duration_re = do {
     my $num = qr{[0-9]+(?:[.,][0-9]+)?};
@@ -68,15 +65,30 @@ sub keywords {
 
   my $formats = +{
     'date-time' => sub {
-      $is_datetime->($_[0]) || (
-        $_[0] =~ m/^(?:\d{4}-\d\d-\d\dT\d\d:\d\d):(\d\d)(?:\.\d+)?(?:[Zz]|[+-]\d\d:\d\d)$/a
-          && $1 eq '60'
-          && do {
+      # https://www.rfc-editor.org/rfc/rfc3339.html#section-5.6
+      $_[0] =~ m/^\d{4}-(\d\d)-(\d\d)T(\d\d):(\d\d):(\d\d)(?:\.\d+)?(?:Z|[+-](\d\d):(\d\d))$/ia
+        && $1 >= 1 && $1 <= 12        # date-month
+        && $2 >= 1 && $2 <= 31        # date-mday
+        && $3 <= 23                   # time-hour
+        && $4 <= 59                   # time-minute
+        && $5 <= 60                   # time-second
+        && (!defined $6 || $6 <= 23)  # time-hour in time-numoffset
+        && (!defined $7 || $7 <= 59)  # time-minute in time-numoffset
+
+        # Time::Moment does month+day sanity check (with leap years), but not leap seconds
+        && ($5 <= 59 && eval { Time::Moment->from_string(uc($_[0])) }
+          || do {
             require DateTime::Format::RFC3339;
             eval { DateTime::Format::RFC3339->parse_datetime($_[0]) };
-          });
+        });
+    },
+    date => sub {
+      # https://www.rfc-editor.org/rfc/rfc3339.html#section-5.6 full-date
+      $_[0] =~ m/^(\d{4})-(\d\d)-(\d\d)$/a
+        && $2 >= 1 && $2 <= 12        # date-month
+        && $3 >= 1 && $3 <= 31        # date-mday
+        && eval { Time::Moment->new(year => $1, month => $2, day => $3) };
     },
-    date => sub { $_[0] =~ /^\d{4}-(?:\d\d)-(?:\d\d)$/a && $is_datetime->($_[0].'T00:00:00Z') },
     time => sub {
       return if $_[0] !~ /^(\d\d):(\d\d):(\d\d)(?:\.\d+)?([Zz]|([+-])(\d\d):(\d\d))$/a
         or $1 > 23
@@ -102,7 +114,7 @@ sub keywords {
     'idn-hostname' => sub { $is_hostname->($idn_decode->($_[0])) },
     ipv4 => $is_ipv4,
     ipv6 => sub {
-      ($_[0] =~ /^(?:[[:xdigit:]]{0,4}:){0,7}[[:xdigit:]]{0,4}$/
+      ($_[0] =~ /^(?:[[:xdigit:]]{0,4}:){0,8}[[:xdigit:]]{0,4}$/
         || $_[0] =~ /^(?:[[:xdigit:]]{0,4}:){1,6}((?:[0-9]{1,3}\.){3}[0-9]{1,3})$/
             && $is_ipv4->($1))
         && $_[0] !~ /:::/
@@ -111,9 +123,11 @@ sub keywords {
         && do {
           my $double_colons = ()= ($_[0] =~ /::/g);
           my $colon_components = grep length, split(/:+/, $_[0], -1);
-          $double_colons < 2 && ($double_colons > 0
-            || ($colon_components == 8 && !defined $1)
-            || ($colon_components == 7 && defined $1))
+          ($double_colons == 1
+            && ((!defined $1 && $colon_components < 8) || (defined $1 && $colon_components < 7)))
+            ||
+          ($double_colons == 0
+            && ((!defined $1 && $colon_components == 8) || (defined $1 && $colon_components == 7)));
         };
     },
     uri => sub {
@@ -199,7 +213,7 @@ JSON::Schema::Modern::Vocabulary::FormatAssertion - Implementation of the JSON S
 
 =head1 VERSION
 
-version 0.568
+version 0.570
 
 =head1 DESCRIPTION
 
